//###########################################################################
//  filename:		  umidi.c
//##########################################################################
// Author:	Copyright (C) 2005 Robert Lang 
// Building on the work of: Reston Condit
//                          Dan Butler
//                          Thorsten Klose
//                          Andrew Smallridge
// Revision:	2.3 
// Date:		January 2005
// Compiled using:	BOOSTC 2.00
//
// Revision History: Change product string when change version
//   1.0  Started with USB Mouse example and modified for BOOSTC and PIC18F2355
//   2.0  Updated descriptors for MIDI and addded additional endpoints. 
//        Change to operate at full and low speed with jumper
//   2.1  Additional interrupts were added for serial port, 64 byte
//        buffers were added.  Debugging logic added.  Note that MIDI    
//        input and output are disabled when "#define DEBUG_SERIAL 	1"
//        Midi output is converted to hex and displayed.
//   2.2  Device will only work at full speed  because low speed does not support
//        bulk transfers.  The device will enumerate at low speed.
//   2.3  Eliminate low speed jumper (since bulk transfers do not work at low speed)
//        and replace with debug jumper.  MIDI serial buffering added.  
//        Jumper out - ascii debug output, Jumper in - midi output
//        "#define DEBUG_SERIAL 	1" eliminated.
// Comments:
// bXXX indicate specific bits in registers defined in umidi.h
// usb buffers are defined in umidi.h
//
//     LED indicators
//    Green	Power
//    Red 0	Reset      
//    Red 1 Addressed, flashing=activity on EP1                
//    Red 2 Configured      
//    Red 3 Sleeping      
//    Red 4 Error
//#########################################################################
// This application is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This application is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this application; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// ==========================================================================
//################################################################################
// include files:
#include "system.h" //pic definition files required by boostc
#include "umidi.h"  //usb definition files required by umidi
#pragma CLOCK_FREQ 24000000  //used by delay_ms
//###############################################################################
// using a 20 Mhz external crystal to generate 24 Mhz CPU clock
#pragma DATA   _CONFIG1L, 00110100b  // USBPPL, CPU divide by 4 , PPL divide by 5                                                          
#pragma DATA   _CONFIG1H, 00001110b  // disable oscillator switchover, 
// disable failsafe clock monitor, HSPPL -ok              
#pragma DATA   _CONFIG2L, 00111111b  // USB voltage regulator enabled,
// brownout set for 2.1 volts, hardware brownout only, PWRT disabled -ok  
#pragma DATA   _CONFIG2H, 00011110b  // watchdog timer disabled -ok      
#pragma DATA   _CONFIG3H, 10000000b  // MCLR enabled -ok                
#pragma DATA   _CONFIG4L, 10000001b  // DEBUG off, disable extended instructons, 
// LVP disabled, enable stack full/underflow reset -ok                 
#pragma DATA   _CONFIG5L, 00001111b  // Read Code protection off -ok  
#pragma DATA   _CONFIG5H, 11000000b  // Read EEPROM and boot block protection off  
#pragma DATA   _CONFIG6L, 00001111b  // Write Code protection off  
#pragma DATA   _CONFIG6H, 11100000b  // Write EEPROM, boot block and
// config register protection off                 
#pragma DATA   _CONFIG7L, 00001111b  // Table read protection off   
#pragma DATA   _CONFIG7H, 01000000b  // Boot block table read protection off
//#####################################################################

unsigned char USB_IsIdle;
unsigned char USB_status_device;
unsigned char USB_Curr_Config; 
unsigned char USB_address_pending;
unsigned char USB_dev_req;
unsigned char USB_Interface [3];
unsigned char USB_USTAT;
unsigned char ACTIVE_BUF;
unsigned char USB_protocol;
unsigned char USWSTAT;
unsigned int USB_PID_ERR   = 0; //error counters
unsigned int USB_CRC5_ERR  = 0;
unsigned int USB_CRC16_ERR = 0;
unsigned int USB_DFN8_ERR  = 0;
unsigned int USB_BTO_ERR   = 0;
unsigned int USB_WRT_ERR   = 0;
unsigned int USB_OWN_ERR   = 0;
unsigned int USB_BTS_ERR   = 0;
unsigned char * EP0_start;  //ep0 pointers
unsigned char * EP0_end;
unsigned char EP0_maxLength; // all USB buffers are defined as 64 Bytes in umidi.h
unsigned char oend=0;
unsigned char ostart=0;
unsigned char iend=0;
unsigned char istart=0;
unsigned char flashcount=0;
volatile bit bsendfull;
unsigned char debug_serial;

typedef struct {
	unsigned char cable;
	unsigned char byte0;
	unsigned char byte1;
	unsigned char byte2;
	unsigned char running_status;
	unsigned char expected_bytes;
	unsigned char wait_bytes;
	unsigned char sysex_ctr;
}midi_rec;
midi_rec midi;
struct BufferDescriptorEntry  // buffer descriptor
{
	unsigned char EPStat;
	unsigned char Bytes;
	unsigned int Address;
};
struct BufferStruct   // buffer structure
{
	unsigned char bmRequestType;
	unsigned char bRequest;
	unsigned int   wValue;
	unsigned int   wIndex;
	unsigned int   wLength;
};
struct BufferDescriptorEntry * BDT= (struct BufferDescriptorEntry *)0x400; 
struct BufferStruct *  Buffer;  //define Buffer as pointer to a BufferStruct
struct BufferStruct  BufferCopy;
struct BufferDescriptorEntry BDTCopy; 

// Special Note: BOOSTC only allow strings to be stored in ROM (program memory), 
// therefore the data arrays that follow (expected_bytes_common, 
// expected_bytes_system, usb_midi_bytes_to_send) were converted to string format. 
rom unsigned char * expected_bytes_common="\2\2\2\2\1\1\2\0";

// number of expected bytes for midi event -1
//	2, // Note On
//	2, // Note Off
//	2, // Poly Pressure
//	2, // Controller
//	1, // Program Change
//	1, // Channel Pressure
//	2, // Pitch Bender
//	0, // System Message - must be zero, so that expected_bytes_system will be used

rom unsigned char * expected_bytes_system="\1\1\2\1\0\0\0\0\0\0\0\0\0\0\0\0";
// Number of expected bytes for a MIDI system event - 1
//	1, // SysEx Begin (endless until SysEx End F7)
//	1, // MTC Data frame
//	2, // Song Position
//	1, // Song Select
//	0, // Reserved
//	0, // Reserved
//	0, // Request Tuning Calibration
//	0, // SysEx End
// Note: just only for documentation, Realtime Messages don't change the running status
//	0, // MIDI Clock
//	0, // MIDI Tick
//	0, // MIDI Start
//	0, // MIDI Continue
//	0, // MIDI Stop
//	0, // Reserved
//	0, // Active Sense
//	0, // Reset                             // 0 1 2 3 4 5 6 7 8 9 A B C D E F
rom unsigned char * usb_midi_bytes_to_send = "\0\0\2\3\3\1\2\3\3\3\3\3\2\2\3\1";
// Number of bytes which should be sent
// 0: invalid/reserved event
// 1: invalid/reserved event
// 2: two-byte system common messages like MTC, Song Select, etc.
// 3: three-byte system common messages like SPP, etc.
// 4: SysEx starts or continues
// 5: Single-byte system common message or sysex sends with following single byte
// 6: SysEx sends with following two bytes
// 7: SysEx sends with following three bytes
// 8: Note Off
// 9: Note On
// a: Poly-Key Press
// b: Control Change
// c: Program Change
// d: Channel Pressure
// e: PitchBend Change
// f: single byte

//string descriptors in unicode format
const char String0 []  = {4,STRING,9,4};
//manufacturer string
const char  String1 [] = {20, STRING,
'M',0,
'i',0,
'c',0,
'r',0,
'o',0,
'c',0,
'h',0,
'i',0,
'p',0};
//product string
const char String2[] = {42,STRING,
'P',0,
'i',0,
'c',0,
'1',0,
'8',0,
'F',0,
'2',0,
'4',0,
'5',0,
'5',0,
' ',0,
'M',0,
'I',0,
'D',0,
'I',0,
' ',0,
'V',0,
'2',0,
'.',0,
'1',0
};
//  The following descriptors are based on the example given by
//  Thorsten Klose on the internet
char DeviceDescriptor [] = {18, // 18 bytes long 
DEVICE, // descriptor type
0x00, 0x02, // USB specification release (2.00)
0x00, // class code
0x00, // subclass code
0x00, // protocol code
64, // maximum packet size 64
0xD8,0x04, // vendor id (04d8) microchip
0x0A,0x00, //product id (000A)
0x00,0x01, // bcd device release number 1.00
0x01, // index to string that describes vendor
0x02, // index to string that describes product
0x00, // index to string that describes serial number (none)
0x01  // number of possible configurations
};
//CONFIG DESCRIPTOR ============================
const char ConfigDescriptor [] = {0x09, // 9 bytes long
CONFIGURATION, // descriptor type
0x53, 0x00, // total length of config, interface, and endpoint descriptors
0x01, // number of interfaces
0x01, // configuration number
0, // index to string that describes configuration (none)
0x80, // configuration attributes 
50, // current consumption in 2mA units (100 mA)
//AUDIO STREAM INTERFACE DESCRIPTOR============
9, // 9 bytes long
INTERFACE, //Interface descriptor
0,  //zero based index of this interface
0,  //alternate setting
2,  //number of endpoints
1,  //interface class =audio
3,  //interface subclass =midistreaming
0,  //interface sub subclass
0,  //interace descriptor string index
// beginning of class specific descriptors
//MIDI STREAMING INTERFACE HEADER DESCRIPTOR=====================
7, // 7 bytes long
CS_INTERFACE, // Class specific interface
1,  //zero based index of this interface
0x00,0x01,  //revision of class
0x41,0x00,  //total size of class specific descriptors - low, high byte
//MIDI IN EMBEDDED DESCRIPTOR==================
6, //6 bytes long
CS_INTERFACE, // Class specific interface
2, //MIDI In jack subtype
1, //Embedded
1, // Jack id
0, // unused
//MIDI IN EXTERIOR DESCRIPTOR==================
6, //6 bytes long
CS_INTERFACE, // Class specific interface
2, //MIDI In jack subtype
2, //External
2, // Jack id
0, // unused
//MIDI OUT EMBEDDED DESCRIPTOR==================
9, //9 bytes long
CS_INTERFACE, // Class specific interface
3, //MIDI Out jack subtype
1, //Embedded
3, // Jack id
1, // number of input pins
2, // id of entity that this pin is connected to
1, // output pin of the entity to which this input is connected
0, // unused
//MIDI OUT EXTERNAL DESCRIPTOR==================
9, //9 bytes long
CS_INTERFACE, // Class specific interface
3, //MIDI Out jack subtype
2, //External
4, // Jack id
1, // number of input pins
1, // id of entity that this pin is connected to
1, // output pin of the entity to which this input is connected
0, // unused
//STANDARD MIDI STREAMING BULK DATA ENDPOINT DESCRIPTOR
0x09, // 9 bytes long
ENDPOINT, // descriptor type
0x01, // out endpoint characteristics ???
0x02, // endpoint attributes  (bulk not shared)
0x40, 0x00, // max packet size =64
0x00, //ignore for bulk
0x00, //unused
0x00, //unused
//CLASS SPECIFIC MIDI STREAMING BULK DATA ENDPOINT DESCRIPTOR
0x05, // 5 bytes long
CS_ENDPOINT, // descriptor type
1, // midi streaming
1, // number of embedded MIDI in jacks
1, // id of embedded MIDIin jack
//STANDARD MIDI STREAMING BULK DATA ENDPOINT DESCRIPTOR
0x09, // 9 bytes long
ENDPOINT, // descriptor type
0x81, // in endpoint characteristics 
0x02, // endpoint attributes  (bulk not shared)
0x40, 0x00, // max packet size =64
0x00, //ignore for bulk
0x00, //unused
0x00, //unused

//CLASS SPECIFIC MIDI STREAMING BULK DATA ENDPOINT DESCRIPTOR
0x05, // 5 bytes long
CS_ENDPOINT, // descriptor type
1, // midi streaming
1, // number of embedded MIDI out jacks
3 // id of embedded MIDI out jack
};
// end of class specific descriptors
//********************************************
// serial port routines
////////////////////////////////////////////////////////////////////////////
//	Initialise the UART
// By Robert Lang
// Based on coding by Andrew Smallridge
////////////////////////////////////////////////////////////////////////////
// BRG_mode - sets the hardware baud rate generator speed mode
// BRG_mode == 0 for low speed mode
// BRG_divisor - set hardware baud rate generator divisor
void UART_Init(unsigned char BRG_mode, unsigned char BRG_divisor)
{    unsigned char brgh;
// initialize buffer variables
      bsendfull=0;
      istart=0;
      iend=0;
      ostart=0;
      oend=0;	
// set data direction	
        set_bit(ddrc,RC7);
		clear_bit(ddrc,RC6);
		// setup the USART baud rate
		// configure baud rate generator mode High vs Low speed
		if (BRG_mode)
			brgh = 4;
		else
			brgh = 0;	
			
		// configure the divisor	
		spbrg = BRG_divisor;
		txsta= brgh;
		txsta= 00100000b + txsta; 
			// 7   Async mode - don't care
			// 6   8 bit selection
			// 5   Enable TX
			// 4   SYNC - Async mode
			// 3   not implemented
			// 2   BRGH - high speed
			// 1-0 don't care	
		rcsta = 10010000b;  // enable the USART receive logic
			// 7   SPEN - Serial Port Enable
			// 6   8 bit selection
			// 5   Don't care
			// 4   CREN - Enable continuous receive
			// 3   disable address detection
			// 2-0 don't care
			
	// init MIDI runtime variables
	midi.cable = 0;
	midi.running_status = 0x00;
	midi.expected_bytes = 0x00;
	midi.wait_bytes = 0x00;
	midi.sysex_ctr = 0x00;
}		
   
void enable_serial_interrupts ()
{     bGIE=0; // disable global interrupts temporarily 
     clear_bit(rcsta,CREN);  // clear continous receive bit
      set_bit(rcsta,CREN);	/* set continous receive bit */  
// enable uart interrupts      
      set_bit(pie1,RCIE); // enable uart receive interrupt    
      set_bit(pie1,TXIE);   // enable uart transmit interrupt  
      set_bit(intcon,PEIE);  // enable peripheral interrupts 
      bGIE=1;  // enable global interrupts        
}
/******************************************************************************/	
// serial_send: send byte of midi data over serial port
// This serial send is by Thorsten Klose and uses buffering
/******************************************************************************/
void serial_send(unsigned char c)  
{		while( bsendfull );
		bGIE=0;	// Disable all Interrupts
		outbuf[(oend++) & (OLEN-1)] = c; //store char in buffer
		bTXIE=1; // enable transmit interrupt to clear out buffer
		if( ((oend ^ ostart) & (OLEN-1)) == 0 )
		bsendfull = 1;
	   	bGIE = 1;	// Enable all Interrupts
} 	
////////////////////////////////////////////////////////////////////////////
// Output a carriage return and linefeed
// Robert Lang
////////////////////////////////////////////////////////////////////////////
void new_line()
{	serial_send(0x0d);
	serial_send(0x0a);}
////////////////////////////////////////////////////////////////////////////
// Output a string
// Robert Lang
////////////////////////////////////////////////////////////////////////////
void puts(char *source)
{	while (*source != 0)  // null marks end of C language string
	serial_send(*source++); // send string one char at a time
    new_line();} // send cr/lf
////////////////////////////////////////////////////////////////////////////
//	This routine outputs an unsigned char as
//  two ascii characters representing its hex value
// By Robert Lang
////////////////////////////////////////////////////////////////////////////	
void hex( unsigned char value)
{
	unsigned char TEST[2];
	char HV;
	unsigned char i;
	TEST[0]=value>>4;  // get high order nibble
	TEST[1]=value & 0x0F;  //mask out high order nibble leaving low
	for (i=0; i<2;i++) //output two hex characters per char
	{
	 switch (TEST[i])  // assign the ascii
	 {
case 1:
	 HV='1';
	 break;
case 2:
	 HV='2';
	 break;
case 3:
	 HV='3';
	 break;
case 4:
	 HV='4';
	 break;
case 5:
	 HV='5';
	 break;
case 6:
	 HV='6';
	 break;
case 7:
	 HV='7';
	 break;
case 8:
	 HV='8';
	 break;
case 9:
	 HV='9';
	 break;	
case 0xA:
	 HV='a';
	 break;
case 0xB:
	 HV='b';
	 break;
case 0xC:
	 HV='c';
	 break;
case 0xD:
	 HV='d';
	 break;
case 0xE:
	 HV='e';
	 break;
case 0xF:
	 HV='f';
	 break;
default:
     HV='0';	 	 	 	 
	}
	serial_send(HV); // output ascii character
	}
	serial_send(' '); // output blank between words 
	}
/******************************************************************************/
/* serial_txbytes_free: returns number of free bytes in tx buffer  */
/******************************************************************************/
unsigned char serial_txbytes_free()
{	if (oend >= ostart)
        return OLEN - (oend - ostart); 		
		else
		return OLEN + (oend - ostart);
}
/******************************************************************************/
/* serial_receive: receives byte from serial port                     */
/******************************************************************************/
unsigned char serial_receive() 
{			if( iend == istart )  // no data in buffer
			return 0x00;
          
	     	return (inbuf[istart++ & (ILEN-1)]); }
//************************************************************************
// This routine takes data from the USB MIDI event and sends to serial port
//************************************************************************
// Based on coding by Thorsten Klose
unsigned char midi_transmit_handler(unsigned char byte0, unsigned char byte1, unsigned char byte2, unsigned char byte3)
{	unsigned char n;
	n = usb_midi_bytes_to_send[byte0 & 0xf];
if (debug_serial)
    n=n*3; // if debugging midi data each byte requires 3 characters 
          // to be output, 2 hex characters and a space.
	if( serial_txbytes_free() < n )
	return 1; // retry
if (debug_serial) 
{
// display midi output as hex
 hex(byte0); //first byte of midi packet (not transmitted)
if( n >= 3 )
	  hex( byte1);
	if( n >= 6 )
	   hex( byte2);
	if( n >= 9 )
	  hex( byte3);
	new_line();  // end of one midi message 
}
else
{
// just send midi data (not in debug mode)
     if( n >= 1 )
	serial_send( byte1);
	if( n >= 2 )
	serial_send( byte2);
	if( n >= 3 )
	serial_send( byte3);
}		 
	return 0; }	
/******************************************************************************/
/* serial_rxbytes: returns number of bytes in rx buffer              */
/******************************************************************************/
unsigned char serial_rxbytes()
{        if  (iend >= istart)  
		return  (iend - istart); 
		else
		return -(iend - istart); 
}

void Flash_Led()
    // Toggle LED 1 every 5 transmissions by PUTEP1 or GETEP1
    // Midi active sensing should cause this to blink constantly
    {
 	//   	
    if (flashcount==0)
    portb=2;
    if (flashcount==5)
    portb=0;
    flashcount=flashcount+1; 
    if (flashcount==11)
    flashcount=0;
    }	

/* ****** */
/* GetEP1 **************************************************************** */
/* Tests the EP1 OUT OWNS bit.  If there is a buffer available to us (data */
/* has been received, it copies the buffer to your buffer and returns the  */
/* number of bytes received.  If no buffer is available, it returns 0      */
/* *********************************************************************** */
unsigned char GetEP1 ()
{	unsigned char n, i, cable, retry;
	unsigned char out1buf[64];
// work done in this area 3/12
	if (((BDT[EP1OUT].EPStat & 0x80) == 0) && (BDT[EP1OUT].Bytes >= 4))  /* do we own the buffer? */
	{   Flash_Led();  // indicate activity on ep1 
		n=BDT[EP1OUT].Bytes;
		for (i = 0; i < n; i++)
		out1buf [i] = bd1oe_buf[i]; // copy to working buffer
		BDT[EP1OUT].EPStat &= 0x40; /* save only the Data 1/0 bit */
		BDT[EP1OUT].EPStat ^= 0x40; /* toggle Data 0/1 bit */		
		BDT[EP1OUT].EPStat |= 0x88;	/* Turn the buffer back over to the SIE */
	if (debug_serial)  puts("EP1out");	
	for (i = 0; i < n; i=i+4) // the following statement tries to send data out the midi serial port until successful
	while (midi_transmit_handler(out1buf[i], out1buf[i+1], out1buf[i+2], out1buf[i+3])) {}
	return BDT[EP1OUT].Bytes;
	}
	else
	return 0;	/* Buffer not available, return true */ 
	}

/* PutEP1 **************************************************************** */
/* Tests the EP1 IN OWNS bit.  If there is a buffer available to us, your  */
/* buffer is copied and turned over to the SIE for transmission on the     */
/* next IN transfer and returns TRUE (1).  If the buffer is not available, */
/* FALSE is returned (0).                                                  */
/* *********************************************************************** */
unsigned char PutEP1 (unsigned char bytes, unsigned char *buffer)
{	unsigned char * tobuffer;
	unsigned char i;

	ddrb=0; //setup b for output
	if ((BDT[EP1IN].EPStat & 0x80) == 0)  /* do we own the buffer? UOWN=0*/
	{   Flash_Led();  // indicate activity on ep1 
		BDT[EP1IN].Bytes = bytes;
		for (i = 0; i < bytes; i++) {
	    bd1ie_buf [i] = buffer[i];
	}
if (debug_serial) {
puts("EP1IN ");
hex(bd1ie_buf[0]);  // output hex representation of first 
hex(bd1ie_buf[1]);  // 2ND
hex(bd1ie_buf[2]);  // 3RD
hex(bd1ie_buf[3]);  // last byte transmitted
new_line();		
                   }
	BDT[EP1IN].EPStat &= 0x40; /* save only the Data 1/0 bit */
	BDT[EP1IN].EPStat ^= 0x40; /* toggle Data 0/1 bit */
	BDT[EP1IN].EPStat |= 0x88; /* release buffer */ 
		return BDT[EP1IN].Bytes;
	}
		return 0;/* Buffer not available, return 0 */ 
}
//************************************************************************
// This routine generates NOTEON and NOTEOFF midi packets from 
// ascii keyboard input (for program testing only)
//************************************************************************
//void generate_midi(unsigned char flag, unsigned char *buf)
//{
//if (flag=='1') buf[1]=0x90; // note on
//else buf[1]=0x80; // noteoff
//buf[0]=buf[1] >> 4;
//buf[2]=0x3c;
//buf[3]=0x64;// buf(0-3) is now loaded with a noteon 
//               //or noteoff packet for middle C
//}
//************************************************************************
// This routine takes serial port data and sends it via  USB 
//************************************************************************
// Based on coding by Thorsten Klose
void midi_receive_handler(midi_rec midix)
{	static unsigned char mybyte, n;
	unsigned char bytecount;
	unsigned char in1buf[4];
	// Is there something in the serial inbuffer and is USB input stream available?
	if( ((BDT[EP1IN].EPStat & 0x80) == 0) && (n=serial_rxbytes()) )
	{
		mybyte = serial_receive();  // get char from input buffer
		
// The following statements allows the user to input an ascii 1 or 0
// from the computer keyboard while in debug_serial mode and a NOTE ON OR NOTE OFF
// for middle C will be sent to computer.  For program testing only
//		if (debug_serial)
//		{ 
//		generate_midi(mybyte,in1buf);
//		bytecount=4;
//		if (ConfiguredUSB()) // only access EP1 if configured
//       while (PutEP1(bytecount,in1buf)==0) ;  // send via usb
//        return;	// exit routine	
//		}
///////////////////////		
		if( mybyte & 0x80 ) // new MIDI status byte?
		{
			if( mybyte < 0xf8 )// is byte < f8 (sysex end or less)
			{
midi.running_status = mybyte; // store running status
midi.expected_bytes = expected_bytes_common[(mybyte >> 4) & 0x7]; // look up expected bytes
if( midi.expected_bytes==0 ) // System Message, take number of bytes from expected_bytes_system[] array
				{
	midi.expected_bytes = expected_bytes_system[mybyte & 0xf]; // look up expected bytes
				if( mybyte == 0xf0 )
					{
midi.byte0 = 0xf0; // sysex begin
midi.sysex_ctr = 0x01;
					}
					else if( mybyte == 0xf7 ) // sysex end
					{
						switch( midi.sysex_ctr )
						{
							case 0:
	in1buf[0] = (midi.cable << 4) | 0x05; // 5: SysEx ends with single byte
								in1buf[1] = 0xf7;
								in1buf[2] = 0x00;
								in1buf[3] = 0x00;
								break;
	                            
							    case 1:
	in1buf[0] = (midi.cable << 4) | 0x06; // 6: SysEx ends with two bytes
								in1buf[1] = midi.byte0;
								in1buf[2] = 0xf7;
								in1buf[3] = 0x00;
								break;
								
							    default:
	in1buf[0] = (midi.cable << 4) | 0x07; // 7: SysEx ends with two bytes
								in1buf[1] = midi.byte0;
								in1buf[2] = midi.byte1;
								in1buf[3] = 0xf7;
							
						}
				
				if (ConfiguredUSB()) // only access EP1 if configured
                while (PutEP1(4,in1buf)==0) ;  // send via usb					
				midi.sysex_ctr = 0x00; // ensure that next F7 will just send F7
				    }
				}
				midi.wait_bytes = midi.expected_bytes;
			}  
			else // byte not less than f8 (realtime messages)
			{
	
		// Realtime messages don't change the running status and can be sent immediately
				in1buf[0] = (midi.cable << 4) | 0x0f;
				in1buf[1] = mybyte;
				in1buf[2] = 0;
				in1buf[3] = 0;
				if (ConfiguredUSB()) // only access EP1 if configured
                while (PutEP1(4,in1buf)==0) ;  // send via usb
			}
		}  
		else // data byte
		{
			if( midi.running_status == 0xf0 ) // part of a sysex message?
			{
				switch( ++midi.sysex_ctr ) // increment byte counter
				{
					case 1:  // load mybyte into proper byte in midi structure
						midi.byte0 = mybyte; 
						break;
					case 2: 
						midi.byte1 = mybyte; 
						break;
					default: // 3
						midi.byte2 = mybyte; 
				// If third byte is loaded then prepare and send packet
			in1buf[0] = (midi.cable << 4) | 0x04; // 4: SysEx starts or continues
						in1buf[1] = midi.byte0;
						in1buf[2] = midi.byte1;
						in1buf[3] = midi.byte2;
					if (ConfiguredUSB()) // only access EP1 if configured
                    while (PutEP1(4,in1buf)==0) ;  // send via usb
				midi.sysex_ctr = 0x00;	// reset and prepare for next packet
				}
			}
			else // not part of sysex message
			{
				if( midi.wait_bytes==0 ) 
				{
				midi.wait_bytes = midi.expected_bytes - 1; // update wait bytes
				}
				else // wait_bytes !=0
					--midi.wait_bytes;  // decrease wait bytes if not zero
					if( midi.expected_bytes == 1 )
				{
				midi.byte1 = mybyte; // if only one byte was expected store it
					midi.byte2 = 0x00;
				}
				else
				{  // expected_bytes !=1
				   if( midi.wait_bytes==0 ) // if wait bytes = 0 
				      midi.byte2 = mybyte; // store in byte 2
				   else  // if wait_bytes>0
				      midi.byte1 = mybyte; // store in byte 1  
				}
	
				if( midi.wait_bytes==0 ) // is wait bytes = 0 now?
				{
					if( (midi.running_status & 0xf0) != 0xf0 )
					{
	   in1buf[0] = (midi.cable << 4) | (midi.running_status >> 4); // common MIDI message
					}
					else
					{
						switch( midi.expected_bytes) 
						{
							case 0: 
		in1buf[0] = (midi.cable << 4) | 0x05; // SysEx common with one byte
								break;
							case 1:
		in1buf[0] = (midi.cable << 4) | 0x02; // SysEx common with two bytes
								break;
							default:
		in1buf[0] = (midi.cable << 4) | 0x03; // SysEx common with three bytes
						}
					}
					in1buf[1] = midi.running_status;
					in1buf[2] = midi.byte1;
					in1buf[3] = midi.byte2;
					if (ConfiguredUSB()) // only access EP1 if configured
                    while (PutEP1(4,in1buf)==0) ;  // send via usb
				}
			}
		}
	}
}
/* ********************************************************************* */
/* Enable Wakeup on interupt and Activity interrupt then put the         */
/* device to sleep to save power.  Activity on the D+/D- lines will      */
/* set the ACTIVITY interrupt, waking up the part.                       */
/* ********************************************************************* */
void USBSleep ()
{	bACTIVITY_E = 1; // enable activity interrupt
	bUIDLE      = 0; //disable idle interrupt
	bSUSPND	   = 1;  // put in suspend mode
	USB_IsIdle  = 1; /* mark device as idle*/ }
	
/* ********************************************************************* */
/* Service the Activity Interrupt.  This is only enabled when the        */
/* device is put to sleep as a result of inactivity on the bus.  This    */
/* code wakes up the part, disables the activity interrupt and reenables */
/* the idle interrupt.                                                   */
/* ********************************************************************* */
void USBActivity ()
{	bACTIVITY   = 0;
	bACTIVITY_E = 0;
	bSUSPND     = 0; //normal operation
	USB_IsIdle  = 0;
	portb=0; //clear sleep led
}

/* ****************************************************************** */
/* USB Reset interrupt triggered (SE0)                                */
/* initialize the Buffer Descriptor Table,                            */
/* Transition to the DEFAULT state,                                   */
/* Set address to 0                                                   */
/* enable the USB                                                     */
/* ****************************************************************** */
void USBReset ()
{	portb = 1;		// Lite led 0 to indicate Reset status
	USB_Curr_Config = 0;
	USB_IsIdle   = 0;
	bTOK_DONE=0; // clear TOK_DNE bit in uir 4 times to 
	bTOK_DONE=0; // clear out the USTAT FIFO.  See Microchip 	
	bTOK_DONE=0; // 18F2455 data sheet.
	bTOK_DONE=0;
	BDT[EP0OUT].EPStat=	0x88;	// EP0 Out buffer (USB OWNS)
	BDT[EP0IN].EPStat= 0x08;    // EP0 In buffer (MCU OWNS)
	BDT[EP1OUT].EPStat=	0x88;	// EP0 Out buffer (USB OWNS)
	BDT[EP1IN].EPStat= 0x08;    // EP0 In buffer (MCU OWNS)	
	uaddr    = 0; // clear address
	uir      = 0; // clear USB interrupt flag
	uie	 = 00000101b; // enable reset and activity interrupt
	// enable the endpoints
	uep0     = ENDPT_CONTROL;
	uep1     = ENDPT_NON_CONTROL;     
	USWSTAT=DEFAULT_STATE;
	USB_status_device = 1;}

/* ******************* */
/* CopyDescriptorToEP0 ************************************************** */
/* copies the next chunk of buffer descriptor over to the EP0 In buffer.  */
/* Inputs:                                                                */
/*    EP0_start - points to first byte of configuration table to transfer */
/*    EP0_end - total number of bytes to transfer                         */
/*    EP0_maxLength - maximum number of bytes that can be sent during     */
/*    a single transfer                                                   */
/*                                                                        */
// toggles the data0/1 bit before setting the UOWN bit over to SIE.
// toggles pingpong bit after sending
/* ********************************************************************** */
void CopyDescriptorToEP0 ( )
{
	unsigned char bufindex;
	bufindex  = 0;
while ((bufindex < EP0_maxLength) && ((unsigned short)EP0_start <(unsigned short) EP0_end))
	{
		bd0ie_buf[bufindex] = *EP0_start;
		++ EP0_start;
		++ bufindex;
	}
	if (bufindex < EP0_maxLength)	/* are we sending a short packet? */
		USB_dev_req = 0;	/* Yes, clear the device reqest */
if (debug_serial){
	hex(bd0ie_buf[0]);  // output hex representation of first and last byte transmitted
	hex(bd0ie_buf[bufindex-1]);		
    hex(bufindex);  //reports bytes transmitted
    new_line();
                }
	BDT[EP0IN].Bytes  = bufindex;	// number of bytes to send
	BDT[EP0IN].EPStat &= 0x40;		// save only the DTS bit 
	BDT[EP0IN].EPStat ^= 0x40;		// toggle data DTS bit 
	BDT[EP0IN].EPStat |= 0x88;		// set OWN and DTSEN bits 
}
/* ******** */
/* Init USB ********************************************************* */
/* Initializes the USB peripheral, sets up the interrupts             */
/* ****************************************************************** */
//
//  A jumper allows the switching from low to full speed USB, however 
//  since low speed does not support bulk transfers, the USB MIDI function
//  only works at full speed but it will enumerate at low speed.
//
void InitUSB ()
{  	ucfg=  00010100b;  
	//use onchip receiver, internal pullup resistors, full
	//            speed and no pingpong buffers
	EP0_maxLength=64; // for full speed
	if (debug_serial) 	puts("Full speed USB");  // debug message      

	//            speed and no pingpong buffers
	uie = 00000001b;	// enable the Reset interrupt ONLY!!
	uir         = 0;	// clear all USB interrupt flags
	pie2= 00100000b ; //enable usb interrupts
	ucon = 01000000b;	// reset pingpong buffers
	ucon = 00001000b;	// unlock pingpong buffers and enable usb
	USB_Curr_Config   = 0;
	USB_status_device = 1;
	char USB_Interface [] = {0,0,0};
	USB_dev_req       = NULL;
	// clear all error counters
	USB_PID_ERR       = 0;
	USB_CRC5_ERR      = 0;
	USB_CRC16_ERR     = 0;
	USB_DFN8_ERR      = 0;
	USB_BTO_ERR       = 0;
	USB_BTS_ERR       = 0;
	intcon  = intcon | 11000000b;	// Enable GIE & PEIE for usb and uart interrupts
	if(debug_serial)
	puts("USB Init");
}
/* ********************************************************************* */
/* This is activated by the STALL bit in the UIR register.  It really    */
/* just tells us that the SIE sent a STALL handshake.  So far, Don't     */
/* see that any action is required.  Clear the bit and move on.          */
/* ********************************************************************* */
void USBStall ()
{	bSTALL = 0; }
/* ****************************************************************** */
/* The SIE detected an error.  This code increments the appropriate   */
/* error counter and clears the flag.                                 */
/* ****************************************************************** */
void Count_Error ()
{	if (bPID_ERR && bPID_ERR_E){
		++ USB_PID_ERR;
		}
	if (bCRC5 && bCRC5_E){
		++ USB_CRC5_ERR;
				}
	if (bCRC16 && bCRC16_E){
		++ USB_CRC16_ERR;
		}
	if (bDFN8 && bDFN8_E){
		++ USB_DFN8_ERR;
		}
	if (bBTO_ERR && bBTO_ERR_E){
		++ USB_BTO_ERR;
		}
	if (bBTS_ERR && bBTS_ERR_E){
		++ USB_BTS_ERR;
		}
	ueir = 0; //Clear all USB error flag bits
	bUERR = 0; /*Clear master USB error flag bit*/ }
	
	
/* ******************************************************************* */
/* Process token done interrupt...  Most of the work gets done through */
/* this interrupt.  Process_Req is signaled in response to an In, Out,  */
/* or Setup transaction.                                               */
/* ******************************************************************* */
void Process_Req ()
{	unsigned char  *UEPArray;
	unsigned char  DescriptorType;
	unsigned char  Endpoint;
	unsigned char  Interface;
	unsigned char  DescriptorID;
	unsigned char  StringID;
	USB_USTAT       = ustat;
	// THE FOLLOWING CALCULATES THE ACTIVE BUFFER FROM CONTENTS OF ustat
	ACTIVE_BUF = (USB_USTAT >>3) *2; // Get endpoint (2 buffers per endpoint with in/out 
	ACTIVE_BUF = ACTIVE_BUF + ((USB_USTAT&0x4)>>2);  //2 buffers per in/ out 
//save data in buffer descriptor table
	BDTCopy.EPStat  = BDT[ACTIVE_BUF].EPStat;
	BDTCopy.Bytes   = BDT[ACTIVE_BUF].Bytes;
	BDTCopy.Address = BDT[ACTIVE_BUF].Address;
	bTOK_DONE        = 0;  // clear token done flag
	if ((BDTCopy.EPStat & 0x3C) == TOKEN_IN)  // TOKEN IN
	{
		if (USB_USTAT == 0x04)
		{ /* Process EP0 In's */
			if (USB_dev_req == GET_DESCRIPTOR) CopyDescriptorToEP0 ();
				}
		else if (USB_USTAT == 0x0C)
		{ /* process EP1 In's */
//	        if (debug_serial) puts("EP1in");
//          if (ConfiguredUSB()) midi_receive_handler(&midi);  
		}
		else
		{ /* process EP2 In's */
		}
	}
	else if ((BDTCopy.EPStat & 0x3C) == TOKEN_OUT)  //TOKEN OUT
	{ 
		if (USB_USTAT == 0x00)
		{ /* process EP0 Out's */
	
		}
		else if (USB_USTAT == 0x08)
		{
		if (ConfiguredUSB()) // do not attempt to get data unless configuration complete	 
		while (GetEP1()==0); // get data if any has been sent	
		}
		else
		{ /* process EP2 Out's */
		}
	}
	else if ((BDTCopy.EPStat & 0x3C) == TOKEN_SETUP)  // TOKEN SETUP
	{
	    Buffer =  (struct BufferStruct *) BDT[EP0OUT].Address;
		BufferCopy.bmRequestType = Buffer->bmRequestType;
		BufferCopy.bRequest      = Buffer->bRequest;
		BufferCopy.wValue        = Buffer->wValue;
		BufferCopy.wIndex        = Buffer->wIndex;
		BufferCopy.wLength       = Buffer->wLength;
        bPID_ERR=0;  // Clear REQUEST ERROR  
		BDT[EP0OUT].Bytes=64;
		if (BufferCopy.bmRequestType == 0x21)
			BDT[EP0OUT].EPStat	= 0xC8;
		else
	    BDT[EP0OUT].EPStat  = 0x88;/* Turn the buffer around, make it available for the SIE */
		BDT[EP0IN].EPStat   = 0x08;  
		bPKT_DIS     = 0;
		USB_dev_req = 0;
		switch (BufferCopy.bmRequestType)
		{
// First bmRequestType ************************************		
		case HOSTTODEVICE:         // 00
	    if (debug_serial) 	puts("H>D");

	
			switch (BufferCopy.bRequest)
			{
//================			
		case CLEAR_FEATURE:	   //01
		/* Remote wakeup is only valid device feature */
				if (BufferCopy.wValue == 1)
				{
					USB_status_device &= 0xFD;
				Send_0Len_pkt;
				}
				else
				    {
				    STALL_EP0;
				    }
				break;
//================
			case SET_FEATURE: //03
			/* Set Device Feature.  Only valid device */
				if ((BufferCopy.wValue & 0xff) == 1)
				{
				USB_status_device |= 0x02;	/* feature is remote wakeup */
					Send_0Len_pkt;
				}
				else
				 {
				 STALL_EP0;
				 }
				break;
//================
			case SET_ADDRESS: //05
			USB_address_pending = BufferCopy.wValue;
	        if (debug_serial) { 
	        puts("SA");
			hex(USB_address_pending); // output the address as ascii (one char)
            new_line(); 
            }			
				if (USB_address_pending < 0x80)
				{
					Send_0Len_pkt;
					USB_dev_req = SET_ADDRESS;
		uie = 00001001b; //enable transaction complete and reset interrupts
				}
				else
				 {
				 STALL_EP0;
				 }
				break;
//================
			case SET_CONFIGURATION:  //09
				if (BufferCopy.wValue <= NUM_CONFIGURATIONS)
					USB_Curr_Config = BufferCopy.wValue;

  if (debug_serial) {
			puts("SC");
			hex(USB_Curr_Config); //output the config 				
                    }
				
	
				if (BufferCopy.wValue == 0)
					{
					USWSTAT= ADDRESS_STATE;
                    if (debug_serial) puts("A");
					}
				else
			     {
					USWSTAT= CONFIG_STATE;
					portb=4; // Lite led 2 to indicate Configured status
                    if (debug_serial) puts("C");
				 }
				Send_0Len_pkt;
	bd1statie = 0x48; //SET MCU AS OWNER OF THIS BUFFER, ENABLE DATA TOGGLE SYNC
				uep1 = ENDPT_NON_CONTROL;
  	          break;
//================
			default:
			    {
         	    STALL_EP0;
         	    }
				break;
			}
			
			break;
// Second bmRequestType ************************************	
		case HOSTTOINTERFACE:   //01
             if (debug_serial) puts("H>I");
			switch (BufferCopy.bRequest)
			{
			case SET_INTERFACE:  //0B
            if (debug_serial) puts("SInter");
				if (USWSTAT == CONFIG_STATE)
				{
					Interface = BufferCopy.wIndex;
					USB_Interface [Interface] = BufferCopy.wValue;
					Send_0Len_pkt;
				}
				else
                {
                STALL_EP0;
                }
				break;
			case CLEAR_FEATURE:   //01
			case SET_FEATURE:	//03
			/* Set Interface feature - Not Valid */
			default:
				STALL_EP0;
				break;
//================				
			}
			break;
// Third bmRequestType ************************************				
		case HOSTTOENDPOINT:
  if (debug_serial)  puts("H>E");
	  
			switch (BufferCopy.bRequest)
			{
//================			
			case CLEAR_FEATURE:  //01
				UEPArray = (unsigned char *) &uep0;
				Endpoint = BufferCopy.wIndex & 0x0F;
	            if (BufferCopy.wValue == 0)  
	            /* Only valid feature is 0 (Remote Wakeup) */
				{
				if (((USWSTAT & 0x03) == ADDRESS_STATE) && (Endpoint == 0))
					{
						UEPArray [Endpoint] &= 0xFE;
						Send_0Len_pkt;
					}
				else if (((USWSTAT& 0x03) == CONFIG_STATE) && (Endpoint < 3))
					{
						UEPArray [Endpoint] &= 0xFE;
						Send_0Len_pkt;
					}
					else
					{
                   	STALL_EP0;
                   	}
				}
				    else
				    {
                  	STALL_EP0;
                  	}
				break;
//================
			case SET_FEATURE:  //03
				UEPArray = (unsigned char *) &uep0;
				Endpoint = BufferCopy.wIndex & 0x0F;
	if (BufferCopy.wValue == 0)  /* Only valid feature is 0 (Remote Wakeup) */
				{
			if (((USWSTAT& 0x03) == ADDRESS_STATE) && (Endpoint == 0))
					{
						UEPArray [Endpoint] |= 1;
						Send_0Len_pkt;
					}
			else if (((USWSTAT& 0x03) == CONFIG_STATE) && (Endpoint < 3))
					{
						UEPArray [Endpoint] |= 1;
						Send_0Len_pkt;
					}
					else
					{
                   	STALL_EP0;
                   	}
				}
				else
			        {
                   	STALL_EP0;
                   	}
				break;
//================
			default:
			{ 
            STALL_EP0;}
			}
			
			break;
// Fourth bmRequestType ************************************	
		case DEVICETOHOST:  //80
  if (debug_serial) puts("D>H");
			switch (BufferCopy.bRequest)
			{
//================			
			case GET_CONFIGURATION:
            if (debug_serial) puts("GC");

			bd0ie_buf [0] = USB_Curr_Config;
				BDT[EP0IN].Bytes = 1;
				BDT[EP0IN].EPStat = 0xc8;
				break;
//================
			case GET_DESCRIPTOR:
                if (debug_serial) puts("GD");
				DescriptorID = (unsigned char) (BufferCopy.wValue >> 8);
				if (DescriptorID == DEVICE)
				
				{   
                if (debug_serial) puts("device");
				USB_dev_req = GET_DESCRIPTOR;
				EP0_start = DeviceDescriptor;
				EP0_end   = DeviceDescriptor + sizeof(DeviceDescriptor);
if (BufferCopy.wLength < ((unsigned short)EP0_end - (unsigned short)EP0_start))
						EP0_end = EP0_start + BufferCopy.wLength;
     					CopyDescriptorToEP0 ();
				}
				else if (DescriptorID == CONFIGURATION)
				{   
                if (debug_serial) puts("config");
				USB_dev_req = GET_DESCRIPTOR;
				EP0_start = ConfigDescriptor;
				EP0_end   = ConfigDescriptor + sizeof(ConfigDescriptor);
if (BufferCopy.wLength < ((unsigned short)EP0_end - (unsigned short)EP0_start))
						EP0_end = EP0_start + BufferCopy.wLength;
  					CopyDescriptorToEP0 ();
				}
				else if (DescriptorID == STRING)
				{
                    if (debug_serial) puts("string");
					StringID = (unsigned char) BufferCopy.wValue;
					USB_dev_req = GET_DESCRIPTOR;
					switch (StringID)
					{
					case 0:
						EP0_start = String0;  // String0 is start of string0
						EP0_end   = String0 + String0[0]; // String[0] is length of String0
if (BufferCopy.wLength < ((unsigned short)EP0_end - (unsigned short)EP0_start))
							EP0_end = EP0_start + BufferCopy.wLength;
			CopyDescriptorToEP0 (); // this copies the whole string in 64 byte pieces
						break;
					case 1:
						EP0_start = String1;
						EP0_end   = String1 + String1[0];
if (BufferCopy.wLength < ((unsigned short)EP0_end - (unsigned short)EP0_start))
							EP0_end = EP0_start + BufferCopy.wLength;
						CopyDescriptorToEP0 ();
						break;
					case 2:
						EP0_start = String2;
						EP0_end   = String2 + String2[0];
if (BufferCopy.wLength < ((unsigned short)EP0_end - (unsigned short)EP0_start))
							EP0_end = EP0_start + BufferCopy.wLength;
						CopyDescriptorToEP0 ();
						break;
// Additional string processing can be added here						
						
					default:
					{
                    if (debug_serial) puts("stall IN");
					STALL_PID_EP0IN;}	/* REQUEST ERROR */
					}
				}
				else
				{
                if (debug_serial) puts("stall IN");
				STALL_PID_EP0IN;	/* REQUEST ERROR */
				}
				break;
//================
			case GET_STATUS:
                if (debug_serial) puts("GS");
				bd0ie_buf[0] = USB_status_device;
				bd0ie_buf[1] = 0;
				BDT [EP0IN].Bytes = 2;
				BDT [EP0IN].EPStat = 0xc8;
				break;
//================
			default:
				break;
			}
//================			
			break;
// Fifth bmRequestType ************************************	
		case INTERFACETOHOST:
            if (debug_serial) puts("I>H");
			switch (BufferCopy.bRequest)
			{
//================			
			case GET_INTERFACE:
                if (debug_serial) puts("GI");
				Interface = BufferCopy.wIndex;
				if ((USWSTAT== CONFIG_STATE) && (Interface < NUM_INTERFACES))
				{
					bd0ie_buf[0] = USB_Interface [Interface];
					BDT [EP0IN].Bytes = 1;
					BDT [EP0IN].EPStat = 0xc8;
				}
				else
				    {
             		STALL_EP0;
             		}
				break;
//================
			case GET_STATUS:
                if (debug_serial) puts("GS");
				bd0ie_buf[1] = 0;
				BDT [EP0IN].Bytes =  2;
				Interface = BufferCopy.wIndex;
				if ((USWSTAT == ADDRESS_STATE) && (Interface == 0))
				{
					bd0ie_buf[0] = USB_Interface [Interface];
					BDT [EP0IN].EPStat = 0xc8;
					
				}
			else if ((USWSTAT == CONFIG_STATE) && (Interface < NUM_INTERFACES))
				{
					bd0ie_buf[0] = USB_Interface [Interface];
					BDT [EP0IN].EPStat = 0xc8;
						}
				else
					{
                  	STALL_EP0;
                  	}
				break;
			case GET_DESCRIPTOR:
				DescriptorType = BufferCopy.wValue >> 8;
				   {
                   STALL_EP0;
                   }	 /* unrecognised request */
				break;
			default:
				break;
			}
			break;
// Sixth bmRequestType ************************************	
		case ENDPOINTTOHOST:
            if (debug_serial) puts("E>H");
			if (BufferCopy.bRequest == GET_STATUS)
			{   
                if (debug_serial) puts("GS");
                UEPArray = (unsigned char *) &uep0;
				Endpoint = BufferCopy.wIndex & 0x0F;
				bd0ie_buf[1] = 0;
				BDT[EP0IN].Bytes = 2;

				if (Endpoint < 3)
				{
					bd0ie_buf[0] = UEPArray [Endpoint] & 0x01;
					BDT[EP0IN].EPStat = 0xc8;
				}
				else
					{
              		STALL_EP0;
              		}
			}
			break;
// Seventh bmRequestType (non defined) ************************************	
		default:
       	STALL_EP0;		
}}}
/* ********************************************************************* */
/* Branch off and service the USB interrupt flags			 */
/* ********************************************************************* */
void ServiceUSB () 
{
	if (bTOK_DONE) 
	  Process_Req();
		
	if (bSTALL) 	{   
        if (debug_serial) puts("Unstall");
		USBStall ();}
			 
	if (bUERR) 	{	
        if (debug_serial) puts("Error");
		Count_Error ();
		portb=16;} // lite led 4 to indicate error
			
	if (bUIDLE)		{	
        if (debug_serial) puts("Sleep");
		USBSleep ();
		portb=8;} // Lite led 3 to indicate sleeping status
}
////////////////////////////////////////////////////////////////
// Interrupt service routine. Branch off to different interrupts
////////////////////////////////////////////////////////////////	
void interrupt (void)
 {
   unsigned char c;
// UART RECEIVE INTERRUPT - HIGHEST PRIORITY	   
 	if ( bRCIE && bRCIF  ) { // UART receive interrupt get processed first

      if (bFERR | bOERR) { // were there framing or overflow errors 
	    portb=16; // if so lite led 4 to indicate error and return
	    return;	
	                     }	
 	c=rcreg; // read char from UART receive register 
 	 //this will also clear receive interrupt flag when read
 	if (istart + ILEN != iend)  // is there room in the input buffer?
 	{   
 		inbuf[iend++ & (ILEN-1)]=c; // load char in input buffer
    }
    return;
 }
// UART TRANSMIT INTERRUPT  - LESSER PRIORITY 	
	if ( bTXIE && bTXIF  ) 
	{ // UART transmit register empty?
 	 	if (ostart != oend)  // is there something in output buffer?
 	        {  // Yes, then send it
    txreg=outbuf[ostart++ & (OLEN-1)]; // load char from buffer to UART transmit register
 		     // this will also set bTXIF after transmission. 
            bsendfull=0; // output buffer is no longer full
  	        }
 	    else  // Nothing to send, disable transmit interrupt
 	   bTXIE=0; 
 	   return;
 	}	 

// USB IS LOWEST PRIORITY	
	if (bUSBIE && bUSBIF) {  // USB interrupt?
		if (bACTIVITY && bACTIVITY_E) // WAS IT AN ACTIVITY WAKEUP?
			USBActivity ();
		if (bUSBRST && bUSBRST_E)// USB reset must be serviced immediately
			USBReset();
		if (bTOK_DONE && bTOK_DONE_E)  // WAS IT A TOKEN DONE 
		{
			if (USB_dev_req == SET_ADDRESS)	// Finish Set Address
			{   				 
				USB_dev_req = NULL;
				USB_Curr_Config = 0;
				uaddr = USB_address_pending;
				uie = 00000001b;		// enable just the reset interrupt
 				if (USB_address_pending > 0)
					{
					USWSTAT= ADDRESS_STATE;
					portb=2; // Lite led 1 to indicate Addressed status
					}
				else
				    {
					USWSTAT= DEFAULT_STATE;
					}
			}
			bTOK_DONE=0; // clear Token Done flag
		}
		bUSBIF=0;		// Clear USB interrupt flag
} 
}
// *****************************************************
//  AT LAST THE MAIN PROGRAM
// *****************************************************
void main() 
{
	unsigned char i;
	unsigned short j;
	const unsigned char led[]={1,2,4,8,16};
	ddrb = 0;
	ddrc=00000001b; //set bit 0 for input
	for (i=0; i<5;i++)
	{
	portb=led[i]; // flash leds to make sure chip is running
	delay_ms( 100 );  // slow down flashes so we can see and also 
	                  // allow time before USB initialization
	}
    if (portc  & 0x01) debug_serial=1; //without jumper J6 ascii debug is on
	else debug_serial=0; // normal midi output
// Setting the serial port baud rate is a bit tricky
// This project uses a 20Mhz crystal, however the system clock
// is actually 24Mhz based on the CPUDIV config bits and desire for
// compatibility with USB.
// serial port will be for debug
//	UART_Init(0,155); // 2400 BAUD @ 24Mhz clock	
//	UART_Init(1,155); // 9600 BAUD @ 24Mhz clock
//	UART_Init(1,77); // 19200 BAUD @ 24Mhz clock
//	UART_Init(1,38); // 38400 BAUD @ 24Mhz clock
//	UART_Init(1,12); // 115200 BAUD @ 24Mhz clock
if (debug_serial) 	UART_Init(1,12); //115200 BAUD @ 24Mhz clock
else    UART_Init(1,47); // MIDI 31250 BAUD
enable_serial_interrupts(); 
if (debug_serial)  puts("USB MIDI Interface V2.3");    
    portb=0; //clear leds
	InitUSB ();		//   allow SIE to come online before beginning USB
					//   initialization
// The program will loop here waiting for something to happen
	while (1)	{	
	ServiceUSB();	// Service USB functions if needed
// nothing should be sent or received on EP1 until device is configured
 if (ConfiguredUSB()) 	{	// Wait until device is configured before using
     midi_receive_handler(midi);  //See if anything is coming in on midi serial line
						}
}
}


